@@ -1,41 +1,42 @@
-#*********************************************************************
-#*
-#* $Id: yocto_anbutton.py 23243 2016-02-23 14:13:12Z seb $
-#*
-#* Implements yFindAnButton(), the high-level API for AnButton functions
-#*
-#* - - - - - - - - - License information: - - - - - - - - - 
-#*
-#*  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
-#*
-#*  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
-#*  non-exclusive license to use, modify, copy and integrate this
-#*  file into your software for the sole purpose of interfacing
-#*  with Yoctopuce products.
-#*
-#*  You may reproduce and distribute copies of this file in
-#*  source or object form, as long as the sole purpose of this
-#*  code is to interface with Yoctopuce products. You must retain
-#*  this notice in the distributed source file.
-#*
-#*  You should refer to Yoctopuce General Terms and Conditions
-#*  for additional information regarding your rights and
-#*  obligations.
-#*
-#*  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
-#*  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING 
-#*  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
-#*  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
-#*  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
-#*  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
-#*  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
-#*  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
-#*  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
-#*  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
-#*  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
-#*  WARRANTY, OR OTHERWISE.
-#*
-#*********************************************************************/
+# -*- coding: utf-8 -*-
+# ********************************************************************
+#
+#  $Id: yocto_anbutton.py 38899 2019-12-20 17:21:03Z mvuilleu $
+#
+#  Implements yFindAnButton(), the high-level API for AnButton functions
+#
+#  - - - - - - - - - License information: - - - - - - - - -
+#
+#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
+#
+#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
+#  non-exclusive license to use, modify, copy and integrate this
+#  file into your software for the sole purpose of interfacing
+#  with Yoctopuce products.
+#
+#  You may reproduce and distribute copies of this file in
+#  source or object form, as long as the sole purpose of this
+#  code is to interface with Yoctopuce products. You must retain
+#  this notice in the distributed source file.
+#
+#  You should refer to Yoctopuce General Terms and Conditions
+#  for additional information regarding your rights and
+#  obligations.
+#
+#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
+#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
+#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
+#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
+#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
+#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
+#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
+#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
+#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
+#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
+#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
+#  WARRANTY, OR OTHERWISE.
+#
+# *********************************************************************
 
 
 __docformat__ = 'restructuredtext en'
@@ -46,7 +47,8 @@ from yoctopuce.yocto_api import *
 #noinspection PyProtectedMember
 class YAnButton(YFunction):
     """
-    Yoctopuce application programming interface allows you to measure the state
+    The YAnButton class provide access to basic resistive inputs.
+    Such inputs can be used to measure the state
     of a simple button as well as to read an analog potentiometer (variable resistance).
     This can be use for instance with a continuous rotating knob, a throttle grip
     or a joystick. The module is capable to calibrate itself on min and max values,
@@ -54,11 +56,13 @@ class YAnButton(YFunction):
     potentiometer position, regardless of its total resistance.
 
     """
-#--- (end of YAnButton class start)
+    #--- (end of YAnButton class start)
     #--- (YAnButton return codes)
     #--- (end of YAnButton return codes)
     #--- (YAnButton dlldef)
     #--- (end of YAnButton dlldef)
+    #--- (YAnButton yapiwrapper)
+    #--- (end of YAnButton yapiwrapper)
     #--- (YAnButton definitions)
     CALIBRATEDVALUE_INVALID = YAPI.INVALID_UINT
     RAWVALUE_INVALID = YAPI.INVALID_UINT
@@ -96,41 +100,30 @@ class YAnButton(YFunction):
         #--- (end of YAnButton attributes)
 
     #--- (YAnButton implementation)
-    def _parseAttr(self, member):
-        if member.name == "calibratedValue":
-            self._calibratedValue = member.ivalue
-            return 1
-        if member.name == "rawValue":
-            self._rawValue = member.ivalue
-            return 1
-        if member.name == "analogCalibration":
-            self._analogCalibration = member.ivalue
-            return 1
-        if member.name == "calibrationMax":
-            self._calibrationMax = member.ivalue
-            return 1
-        if member.name == "calibrationMin":
-            self._calibrationMin = member.ivalue
-            return 1
-        if member.name == "sensitivity":
-            self._sensitivity = member.ivalue
-            return 1
-        if member.name == "isPressed":
-            self._isPressed = member.ivalue
-            return 1
-        if member.name == "lastTimePressed":
-            self._lastTimePressed = member.ivalue
-            return 1
-        if member.name == "lastTimeReleased":
-            self._lastTimeReleased = member.ivalue
-            return 1
-        if member.name == "pulseCounter":
-            self._pulseCounter = member.ivalue
-            return 1
-        if member.name == "pulseTimer":
-            self._pulseTimer = member.ivalue
-            return 1
-        super(YAnButton, self)._parseAttr(member)
+    def _parseAttr(self, json_val):
+        if json_val.has("calibratedValue"):
+            self._calibratedValue = json_val.getInt("calibratedValue")
+        if json_val.has("rawValue"):
+            self._rawValue = json_val.getInt("rawValue")
+        if json_val.has("analogCalibration"):
+            self._analogCalibration = (json_val.getInt("analogCalibration") > 0 if 1 else 0)
+        if json_val.has("calibrationMax"):
+            self._calibrationMax = json_val.getInt("calibrationMax")
+        if json_val.has("calibrationMin"):
+            self._calibrationMin = json_val.getInt("calibrationMin")
+        if json_val.has("sensitivity"):
+            self._sensitivity = json_val.getInt("sensitivity")
+        if json_val.has("isPressed"):
+            self._isPressed = (json_val.getInt("isPressed") > 0 if 1 else 0)
+        if json_val.has("lastTimePressed"):
+            self._lastTimePressed = json_val.getLong("lastTimePressed")
+        if json_val.has("lastTimeReleased"):
+            self._lastTimeReleased = json_val.getLong("lastTimeReleased")
+        if json_val.has("pulseCounter"):
+            self._pulseCounter = json_val.getLong("pulseCounter")
+        if json_val.has("pulseTimer"):
+            self._pulseTimer = json_val.getLong("pulseTimer")
+        super(YAnButton, self)._parseAttr(json_val)
 
     def get_calibratedValue(self):
         """
@@ -140,10 +133,12 @@ class YAnButton(YFunction):
 
         On failure, throws an exception or returns YAnButton.CALIBRATEDVALUE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YAnButton.CALIBRATEDVALUE_INVALID
-        return self._calibratedValue
+        res = self._calibratedValue
+        return res
 
     def get_rawValue(self):
         """
@@ -153,10 +148,12 @@ class YAnButton(YFunction):
 
         On failure, throws an exception or returns YAnButton.RAWVALUE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YAnButton.RAWVALUE_INVALID
-        return self._rawValue
+        res = self._rawValue
+        return res
 
     def get_analogCalibration(self):
         """
@@ -166,10 +163,12 @@ class YAnButton(YFunction):
 
         On failure, throws an exception or returns YAnButton.ANALOGCALIBRATION_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YAnButton.ANALOGCALIBRATION_INVALID
-        return self._analogCalibration
+        res = self._analogCalibration
+        return res
 
     def set_analogCalibration(self, newval):
         """
@@ -194,10 +193,12 @@ class YAnButton(YFunction):
 
         On failure, throws an exception or returns YAnButton.CALIBRATIONMAX_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YAnButton.CALIBRATIONMAX_INVALID
-        return self._calibrationMax
+        res = self._calibrationMax
+        return res
 
     def set_calibrationMax(self, newval):
         """
@@ -225,10 +226,12 @@ class YAnButton(YFunction):
 
         On failure, throws an exception or returns YAnButton.CALIBRATIONMIN_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YAnButton.CALIBRATIONMIN_INVALID
-        return self._calibrationMin
+        res = self._calibrationMin
+        return res
 
     def set_calibrationMin(self, newval):
         """
@@ -256,10 +259,12 @@ class YAnButton(YFunction):
 
         On failure, throws an exception or returns YAnButton.SENSITIVITY_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YAnButton.SENSITIVITY_INVALID
-        return self._sensitivity
+        res = self._sensitivity
+        return res
 
     def set_sensitivity(self, newval):
         """
@@ -289,10 +294,12 @@ class YAnButton(YFunction):
 
         On failure, throws an exception or returns YAnButton.ISPRESSED_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YAnButton.ISPRESSED_INVALID
-        return self._isPressed
+        res = self._isPressed
+        return res
 
     def get_lastTimePressed(self):
         """
@@ -305,10 +312,12 @@ class YAnButton(YFunction):
 
         On failure, throws an exception or returns YAnButton.LASTTIMEPRESSED_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YAnButton.LASTTIMEPRESSED_INVALID
-        return self._lastTimePressed
+        res = self._lastTimePressed
+        return res
 
     def get_lastTimeReleased(self):
         """
@@ -321,23 +330,29 @@ class YAnButton(YFunction):
 
         On failure, throws an exception or returns YAnButton.LASTTIMERELEASED_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YAnButton.LASTTIMERELEASED_INVALID
-        return self._lastTimeReleased
+        res = self._lastTimeReleased
+        return res
 
     def get_pulseCounter(self):
         """
-        Returns the pulse counter value.
+        Returns the pulse counter value. The value is a 32 bit integer. In case
+        of overflow (>=2^32), the counter will wrap. To reset the counter, just
+        call the resetCounter() method.
 
         @return an integer corresponding to the pulse counter value
 
         On failure, throws an exception or returns YAnButton.PULSECOUNTER_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YAnButton.PULSECOUNTER_INVALID
-        return self._pulseCounter
+        res = self._pulseCounter
+        return res
 
     def set_pulseCounter(self, newval):
         rest_val = str(newval)
@@ -351,10 +366,12 @@ class YAnButton(YFunction):
 
         On failure, throws an exception or returns YAnButton.PULSETIMER_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YAnButton.PULSETIMER_INVALID
-        return self._pulseTimer
+        res = self._pulseTimer
+        return res
 
     @staticmethod
     def FindAnButton(func):
@@ -377,7 +394,12 @@ class YAnButton(YFunction):
         found is returned. The search is performed first by hardware name,
         then by logical name.
 
-        @param func : a string that uniquely characterizes the analog input
+        If a call to this object's is_online() method returns FALSE although
+        you are certain that the matching device is plugged, make sure that you did
+        call registerHub() at application initialization time.
+
+        @param func : a string that uniquely characterizes the analog input, for instance
+                YBUZZER2.anButton1.
 
         @return a YAnButton object allowing you to drive the analog input.
         """
@@ -401,6 +423,9 @@ class YAnButton(YFunction):
     def nextAnButton(self):
         """
         Continues the enumeration of analog inputs started using yFirstAnButton().
+        Caution: You can't make any assumption about the returned analog inputs order.
+        If you want to find a specific an analog input, use AnButton.findAnButton()
+        and a hardwareID or a logical name.
 
         @return a pointer to a YAnButton object, corresponding to
                 an analog input currently online, or a None pointer
@@ -415,7 +440,7 @@ class YAnButton(YFunction):
 
 #--- (end of YAnButton implementation)
 
-#--- (AnButton functions)
+#--- (YAnButton functions)
 
     @staticmethod
     def FirstAnButton():
@@ -449,4 +474,4 @@ class YAnButton(YFunction):
 
         return YAnButton.FindAnButton(serialRef.value + "." + funcIdRef.value)
 
-#--- (end of AnButton functions)
+#--- (end of YAnButton functions)
