@@ -1,6 +1,6 @@
 #*********************************************************************
 #*
-#* $Id: yocto_cellular.py 24465 2016-05-12 07:30:46Z mvuilleu $
+#* $Id: yocto_cellular.py 40298 2020-05-05 08:37:49Z seb $
 #*
 #* Implements yFindCellular(), the high-level API for Cellular functions
 #*
@@ -45,7 +45,13 @@ from yoctopuce.yocto_api import *
 #--- (generated code: YCellRecord class start)
 #noinspection PyProtectedMember
 class YCellRecord(object):
-#--- (end of generated code: YCellRecord class start)
+    """
+    YCellRecord objects are used to describe a wireless network.
+    These objects are used in particular in conjunction with the
+    YCellular class.
+
+    """
+    #--- (end of generated code: YCellRecord class start)
     #--- (generated code: YCellRecord return codes)
     #--- (end of generated code: YCellRecord return codes)
     #--- (generated code: YCellRecord dlldef)
@@ -73,41 +79,82 @@ class YCellRecord(object):
 
 #--- (generated code: YCellRecord implementation)
     def get_cellOperator(self):
+        """
+        Returns the name of the the cell operator, as received from the network.
+
+        @return a string with the name of the the cell operator.
+        """
         return self._oper
 
     def get_mobileCountryCode(self):
+        """
+        Returns the Mobile Country Code (MCC). The MCC is a unique identifier for each country.
+
+        @return an integer corresponding to the Mobile Country Code (MCC).
+        """
         return self._mcc
 
     def get_mobileNetworkCode(self):
+        """
+        Returns the Mobile Network Code (MNC). The MNC is a unique identifier for each phone
+        operator within a country.
+
+        @return an integer corresponding to the Mobile Network Code (MNC).
+        """
         return self._mnc
 
     def get_locationAreaCode(self):
+        """
+        Returns the Location Area Code (LAC). The LAC is a unique identifier for each
+        place within a country.
+
+        @return an integer corresponding to the Location Area Code (LAC).
+        """
         return self._lac
 
     def get_cellId(self):
+        """
+        Returns the Cell ID. The Cell ID is a unique identifier for each
+        base transmission station within a LAC.
+
+        @return an integer corresponding to the Cell Id.
+        """
         return self._cid
 
     def get_signalStrength(self):
+        """
+        Returns the signal strength, measured in dBm.
+
+        @return an integer corresponding to the signal strength.
+        """
         return self._dbm
 
     def get_timingAdvance(self):
+        """
+        Returns the Timing Advance (TA). The TA corresponds to the time necessary
+        for the signal to reach the base station from the device.
+        Each increment corresponds about to 550m of distance.
+
+        @return an integer corresponding to the Timing Advance (TA).
+        """
         return self._tad
 
 #--- (end of generated code: YCellRecord implementation)
 
-#--- (generated code: CellRecord functions)
-#--- (end of generated code: CellRecord functions)
+#--- (generated code: YCellRecord functions)
+#--- (end of generated code: YCellRecord functions)
 
 
 #--- (generated code: YCellular class start)
 #noinspection PyProtectedMember
 class YCellular(YFunction):
     """
-    YCellular functions provides control over cellular network parameters
+    The YCellular class provides control over cellular network parameters
     and status for devices that are GSM-enabled.
+    Note that TCP/IP parameters are configured separately, using class YNetwork.
 
     """
-#--- (end of generated code: YCellular class start)
+    #--- (end of generated code: YCellular class start)
     #--- (generated code: YCellular return codes)
     #--- (end of generated code: YCellular return codes)
     #--- (generated code: YCellular dlldef)
@@ -119,10 +166,13 @@ class YCellular(YFunction):
     IMSI_INVALID = YAPI.INVALID_STRING
     MESSAGE_INVALID = YAPI.INVALID_STRING
     PIN_INVALID = YAPI.INVALID_STRING
+    RADIOCONFIG_INVALID = YAPI.INVALID_STRING
     LOCKEDOPERATOR_INVALID = YAPI.INVALID_STRING
     APN_INVALID = YAPI.INVALID_STRING
     APNSECRET_INVALID = YAPI.INVALID_STRING
     PINGINTERVAL_INVALID = YAPI.INVALID_UINT
+    DATASENT_INVALID = YAPI.INVALID_UINT
+    DATARECEIVED_INVALID = YAPI.INVALID_UINT
     COMMAND_INVALID = YAPI.INVALID_STRING
     CELLTYPE_GPRS = 0
     CELLTYPE_EGPRS = 1
@@ -130,6 +180,9 @@ class YCellular(YFunction):
     CELLTYPE_HSDPA = 3
     CELLTYPE_NONE = 4
     CELLTYPE_CDMA = 5
+    CELLTYPE_LTE_M = 6
+    CELLTYPE_NB_IOT = 7
+    CELLTYPE_EC_GSM_IOT = 8
     CELLTYPE_INVALID = -1
     AIRPLANEMODE_OFF = 0
     AIRPLANEMODE_ON = 1
@@ -137,6 +190,7 @@ class YCellular(YFunction):
     ENABLEDATA_HOMENETWORK = 0
     ENABLEDATA_ROAMING = 1
     ENABLEDATA_NEVER = 2
+    ENABLEDATA_NEUTRALITY = 3
     ENABLEDATA_INVALID = -1
     #--- (end of generated code: YCellular definitions)
 
@@ -152,60 +206,55 @@ class YCellular(YFunction):
         self._imsi = YCellular.IMSI_INVALID
         self._message = YCellular.MESSAGE_INVALID
         self._pin = YCellular.PIN_INVALID
+        self._radioConfig = YCellular.RADIOCONFIG_INVALID
         self._lockedOperator = YCellular.LOCKEDOPERATOR_INVALID
         self._airplaneMode = YCellular.AIRPLANEMODE_INVALID
         self._enableData = YCellular.ENABLEDATA_INVALID
         self._apn = YCellular.APN_INVALID
         self._apnSecret = YCellular.APNSECRET_INVALID
         self._pingInterval = YCellular.PINGINTERVAL_INVALID
+        self._dataSent = YCellular.DATASENT_INVALID
+        self._dataReceived = YCellular.DATARECEIVED_INVALID
         self._command = YCellular.COMMAND_INVALID
         #--- (end of generated code: YCellular attributes)
 
     #--- (generated code: YCellular implementation)
-    def _parseAttr(self, member):
-        if member.name == "linkQuality":
-            self._linkQuality = member.ivalue
-            return 1
-        if member.name == "cellOperator":
-            self._cellOperator = member.svalue
-            return 1
-        if member.name == "cellIdentifier":
-            self._cellIdentifier = member.svalue
-            return 1
-        if member.name == "cellType":
-            self._cellType = member.ivalue
-            return 1
-        if member.name == "imsi":
-            self._imsi = member.svalue
-            return 1
-        if member.name == "message":
-            self._message = member.svalue
-            return 1
-        if member.name == "pin":
-            self._pin = member.svalue
-            return 1
-        if member.name == "lockedOperator":
-            self._lockedOperator = member.svalue
-            return 1
-        if member.name == "airplaneMode":
-            self._airplaneMode = member.ivalue
-            return 1
-        if member.name == "enableData":
-            self._enableData = member.ivalue
-            return 1
-        if member.name == "apn":
-            self._apn = member.svalue
-            return 1
-        if member.name == "apnSecret":
-            self._apnSecret = member.svalue
-            return 1
-        if member.name == "pingInterval":
-            self._pingInterval = member.ivalue
-            return 1
-        if member.name == "command":
-            self._command = member.svalue
-            return 1
-        super(YCellular, self)._parseAttr(member)
+    def _parseAttr(self, json_val):
+        if json_val.has("linkQuality"):
+            self._linkQuality = json_val.getInt("linkQuality")
+        if json_val.has("cellOperator"):
+            self._cellOperator = json_val.getString("cellOperator")
+        if json_val.has("cellIdentifier"):
+            self._cellIdentifier = json_val.getString("cellIdentifier")
+        if json_val.has("cellType"):
+            self._cellType = json_val.getInt("cellType")
+        if json_val.has("imsi"):
+            self._imsi = json_val.getString("imsi")
+        if json_val.has("message"):
+            self._message = json_val.getString("message")
+        if json_val.has("pin"):
+            self._pin = json_val.getString("pin")
+        if json_val.has("radioConfig"):
+            self._radioConfig = json_val.getString("radioConfig")
+        if json_val.has("lockedOperator"):
+            self._lockedOperator = json_val.getString("lockedOperator")
+        if json_val.has("airplaneMode"):
+            self._airplaneMode = (json_val.getInt("airplaneMode") > 0 if 1 else 0)
+        if json_val.has("enableData"):
+            self._enableData = json_val.getInt("enableData")
+        if json_val.has("apn"):
+            self._apn = json_val.getString("apn")
+        if json_val.has("apnSecret"):
+            self._apnSecret = json_val.getString("apnSecret")
+        if json_val.has("pingInterval"):
+            self._pingInterval = json_val.getInt("pingInterval")
+        if json_val.has("dataSent"):
+            self._dataSent = json_val.getInt("dataSent")
+        if json_val.has("dataReceived"):
+            self._dataReceived = json_val.getInt("dataReceived")
+        if json_val.has("command"):
+            self._command = json_val.getString("command")
+        super(YCellular, self)._parseAttr(json_val)
 
     def get_linkQuality(self):
         """
@@ -215,10 +264,12 @@ class YCellular(YFunction):
 
         On failure, throws an exception or returns YCellular.LINKQUALITY_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YCellular.LINKQUALITY_INVALID
-        return self._linkQuality
+        res = self._linkQuality
+        return res
 
     def get_cellOperator(self):
         """
@@ -228,10 +279,12 @@ class YCellular(YFunction):
 
         On failure, throws an exception or returns YCellular.CELLOPERATOR_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YCellular.CELLOPERATOR_INVALID
-        return self._cellOperator
+        res = self._cellOperator
+        return res
 
     def get_cellIdentifier(self):
         """
@@ -242,41 +295,48 @@ class YCellular(YFunction):
 
         On failure, throws an exception or returns YCellular.CELLIDENTIFIER_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YCellular.CELLIDENTIFIER_INVALID
-        return self._cellIdentifier
+        res = self._cellIdentifier
+        return res
 
     def get_cellType(self):
         """
         Active cellular connection type.
 
         @return a value among YCellular.CELLTYPE_GPRS, YCellular.CELLTYPE_EGPRS, YCellular.CELLTYPE_WCDMA,
-        YCellular.CELLTYPE_HSDPA, YCellular.CELLTYPE_NONE and YCellular.CELLTYPE_CDMA
+        YCellular.CELLTYPE_HSDPA, YCellular.CELLTYPE_NONE, YCellular.CELLTYPE_CDMA,
+        YCellular.CELLTYPE_LTE_M, YCellular.CELLTYPE_NB_IOT and YCellular.CELLTYPE_EC_GSM_IOT
 
         On failure, throws an exception or returns YCellular.CELLTYPE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YCellular.CELLTYPE_INVALID
-        return self._cellType
+        res = self._cellType
+        return res
 
     def get_imsi(self):
         """
-        Returns an opaque string if a PIN code has been configured in the device to access
-        the SIM card, or an empty string if none has been configured or if the code provided
-        was rejected by the SIM card.
+        Returns the International Mobile Subscriber Identity (MSI) that uniquely identifies
+        the SIM card. The first 3 digits represent the mobile country code (MCC), which
+        is followed by the mobile network code (MNC), either 2-digit (European standard)
+        or 3-digit (North American standard)
 
-        @return a string corresponding to an opaque string if a PIN code has been configured in the device to access
-                the SIM card, or an empty string if none has been configured or if the code provided
-                was rejected by the SIM card
+        @return a string corresponding to the International Mobile Subscriber Identity (MSI) that uniquely identifies
+                the SIM card
 
         On failure, throws an exception or returns YCellular.IMSI_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YCellular.IMSI_INVALID
-        return self._imsi
+        res = self._imsi
+        return res
 
     def get_message(self):
         """
@@ -286,10 +346,12 @@ class YCellular(YFunction):
 
         On failure, throws an exception or returns YCellular.MESSAGE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YCellular.MESSAGE_INVALID
-        return self._message
+        res = self._message
+        return res
 
     def get_pin(self):
         """
@@ -303,10 +365,12 @@ class YCellular(YFunction):
 
         On failure, throws an exception or returns YCellular.PIN_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YCellular.PIN_INVALID
-        return self._pin
+        res = self._pin
+        return res
 
     def set_pin(self, newval):
         """
@@ -331,6 +395,46 @@ class YCellular(YFunction):
         rest_val = newval
         return self._setAttr("pin", rest_val)
 
+    def get_radioConfig(self):
+        """
+        Returns the type of protocol used over the serial line, as a string.
+        Possible values are "Line" for ASCII messages separated by CR and/or LF,
+        "Frame:[timeout]ms" for binary messages separated by a delay time,
+        "Char" for a continuous ASCII stream or
+        "Byte" for a continuous binary stream.
+
+        @return a string corresponding to the type of protocol used over the serial line, as a string
+
+        On failure, throws an exception or returns YCellular.RADIOCONFIG_INVALID.
+        """
+        # res
+        if self._cacheExpiration <= YAPI.GetTickCount():
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
+                return YCellular.RADIOCONFIG_INVALID
+        res = self._radioConfig
+        return res
+
+    def set_radioConfig(self, newval):
+        """
+        Changes the type of protocol used over the serial line.
+        Possible values are "Line" for ASCII messages separated by CR and/or LF,
+        "Frame:[timeout]ms" for binary messages separated by a delay time,
+        "Char" for a continuous ASCII stream or
+        "Byte" for a continuous binary stream.
+        The suffix "/[wait]ms" can be added to reduce the transmit rate so that there
+        is always at lest the specified number of milliseconds between each bytes sent.
+        Remember to call the saveToFlash() method of the module if the
+        modification must be kept.
+
+        @param newval : a string corresponding to the type of protocol used over the serial line
+
+        @return YAPI.SUCCESS if the call succeeds.
+
+        On failure, throws an exception or returns a negative error code.
+        """
+        rest_val = newval
+        return self._setAttr("radioConfig", rest_val)
+
     def get_lockedOperator(self):
         """
         Returns the name of the only cell operator to use if automatic choice is disabled,
@@ -343,16 +447,20 @@ class YCellular(YFunction):
 
         On failure, throws an exception or returns YCellular.LOCKEDOPERATOR_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YCellular.LOCKEDOPERATOR_INVALID
-        return self._lockedOperator
+        res = self._lockedOperator
+        return res
 
     def set_lockedOperator(self, newval):
         """
         Changes the name of the cell operator to be used. If the name is an empty
         string, the choice will be made automatically based on the SIM card. Otherwise,
         the selected operator is the only one that will be used.
+        Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
 
         @param newval : a string corresponding to the name of the cell operator to be used
 
@@ -372,10 +480,12 @@ class YCellular(YFunction):
 
         On failure, throws an exception or returns YCellular.AIRPLANEMODE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YCellular.AIRPLANEMODE_INVALID
-        return self._airplaneMode
+        res = self._airplaneMode
+        return res
 
     def set_airplaneMode(self, newval):
         """
@@ -396,15 +506,18 @@ class YCellular(YFunction):
         Returns the condition for enabling IP data services (GPRS).
         When data services are disabled, SMS are the only mean of communication.
 
-        @return a value among YCellular.ENABLEDATA_HOMENETWORK, YCellular.ENABLEDATA_ROAMING and
-        YCellular.ENABLEDATA_NEVER corresponding to the condition for enabling IP data services (GPRS)
+        @return a value among YCellular.ENABLEDATA_HOMENETWORK, YCellular.ENABLEDATA_ROAMING,
+        YCellular.ENABLEDATA_NEVER and YCellular.ENABLEDATA_NEUTRALITY corresponding to the condition for
+        enabling IP data services (GPRS)
 
         On failure, throws an exception or returns YCellular.ENABLEDATA_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YCellular.ENABLEDATA_INVALID
-        return self._enableData
+        res = self._enableData
+        return res
 
     def set_enableData(self, newval):
         """
@@ -414,9 +527,12 @@ class YCellular(YFunction):
         on roaming networks may cause prohibitive communication costs !
 
         When data services are disabled, SMS are the only mean of communication.
+        Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
 
-        @param newval : a value among YCellular.ENABLEDATA_HOMENETWORK, YCellular.ENABLEDATA_ROAMING and
-        YCellular.ENABLEDATA_NEVER corresponding to the condition for enabling IP data services (GPRS)
+        @param newval : a value among YCellular.ENABLEDATA_HOMENETWORK, YCellular.ENABLEDATA_ROAMING,
+        YCellular.ENABLEDATA_NEVER and YCellular.ENABLEDATA_NEUTRALITY corresponding to the condition for
+        enabling IP data services (GPRS)
 
         @return YAPI.SUCCESS if the call succeeds.
 
@@ -434,15 +550,19 @@ class YCellular(YFunction):
 
         On failure, throws an exception or returns YCellular.APN_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YCellular.APN_INVALID
-        return self._apn
+        res = self._apn
+        return res
 
     def set_apn(self, newval):
         """
         Returns the Access Point Name (APN) to be used, if needed.
         When left blank, the APN suggested by the cell operator will be used.
+        Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
 
         @param newval : a string
 
@@ -464,10 +584,12 @@ class YCellular(YFunction):
 
         On failure, throws an exception or returns YCellular.APNSECRET_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YCellular.APNSECRET_INVALID
-        return self._apnSecret
+        res = self._apnSecret
+        return res
 
     def set_apnSecret(self, newval):
         rest_val = newval
@@ -481,14 +603,18 @@ class YCellular(YFunction):
 
         On failure, throws an exception or returns YCellular.PINGINTERVAL_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YCellular.PINGINTERVAL_INVALID
-        return self._pingInterval
+        res = self._pingInterval
+        return res
 
     def set_pingInterval(self, newval):
         """
         Changes the automated connectivity check interval, in seconds.
+        Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
 
         @param newval : an integer corresponding to the automated connectivity check interval, in seconds
 
@@ -499,11 +625,69 @@ class YCellular(YFunction):
         rest_val = str(newval)
         return self._setAttr("pingInterval", rest_val)
 
-    def get_command(self):
+    def get_dataSent(self):
+        """
+        Returns the number of bytes sent so far.
+
+        @return an integer corresponding to the number of bytes sent so far
+
+        On failure, throws an exception or returns YCellular.DATASENT_INVALID.
+        """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
+                return YCellular.DATASENT_INVALID
+        res = self._dataSent
+        return res
+
+    def set_dataSent(self, newval):
+        """
+        Changes the value of the outgoing data counter.
+
+        @param newval : an integer corresponding to the value of the outgoing data counter
+
+        @return YAPI.SUCCESS if the call succeeds.
+
+        On failure, throws an exception or returns a negative error code.
+        """
+        rest_val = str(newval)
+        return self._setAttr("dataSent", rest_val)
+
+    def get_dataReceived(self):
+        """
+        Returns the number of bytes received so far.
+
+        @return an integer corresponding to the number of bytes received so far
+
+        On failure, throws an exception or returns YCellular.DATARECEIVED_INVALID.
+        """
+        # res
+        if self._cacheExpiration <= YAPI.GetTickCount():
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
+                return YCellular.DATARECEIVED_INVALID
+        res = self._dataReceived
+        return res
+
+    def set_dataReceived(self, newval):
+        """
+        Changes the value of the incoming data counter.
+
+        @param newval : an integer corresponding to the value of the incoming data counter
+
+        @return YAPI.SUCCESS if the call succeeds.
+
+        On failure, throws an exception or returns a negative error code.
+        """
+        rest_val = str(newval)
+        return self._setAttr("dataReceived", rest_val)
+
+    def get_command(self):
+        # res
+        if self._cacheExpiration <= YAPI.GetTickCount():
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YCellular.COMMAND_INVALID
-        return self._command
+        res = self._command
+        return res
 
     def set_command(self, newval):
         rest_val = newval
@@ -530,7 +714,12 @@ class YCellular(YFunction):
         found is returned. The search is performed first by hardware name,
         then by logical name.
 
-        @param func : a string that uniquely characterizes the cellular interface
+        If a call to this object's is_online() method returns FALSE although
+        you are certain that the matching device is plugged, make sure that you did
+        call registerHub() at application initialization time.
+
+        @param func : a string that uniquely characterizes the cellular interface, for instance
+                YHUBGSM1.cellular.
 
         @return a YCellular object allowing you to drive the cellular interface.
         """
@@ -544,7 +733,7 @@ class YCellular(YFunction):
     def sendPUK(self, puk, newPin):
         """
         Sends a PUK code to unlock the SIM card after three failed PIN code attempts, and
-        setup a new PIN into the SIM card. Only ten consecutives tentatives are permitted:
+        setup a new PIN into the SIM card. Only ten consecutive tentatives are permitted:
         after that, the SIM card will be blocked permanently without any mean of recovery
         to use it again. Note that after calling this method, you have usually to invoke
         method set_pin() to tell the YoctoHub which PIN to use in the future.
@@ -560,6 +749,7 @@ class YCellular(YFunction):
         gsmMsg = self.get_message()
         if not ((gsmMsg)[0: 0 + 13] == "Enter SIM PUK"):
             self._throw(YAPI.INVALID_ARGUMENT, "PUK not expected at this time")
+            return YAPI.INVALID_ARGUMENT
         if newPin == "":
             return self.set_command("AT+CPIN=" + puk + ",0000;+CLCK=SC,0,0000")
         return self.set_command("AT+CPIN=" + puk + "," + newPin)
@@ -578,6 +768,22 @@ class YCellular(YFunction):
         """
         return self.set_apnSecret("" + username + "," + password)
 
+    def clearDataCounters(self):
+        """
+        Clear the transmitted data counters.
+
+        @return YAPI.SUCCESS when the call succeeds.
+
+        On failure, throws an exception or returns a negative error code.
+        """
+        # retcode
+
+        retcode = self.set_dataReceived(0)
+        if retcode != YAPI.SUCCESS:
+            return retcode
+        retcode = self.set_dataSent(0)
+        return retcode
+
     def _AT(self, cmd):
         """
         Sends an AT command to the GSM module and returns the command output.
@@ -622,7 +828,6 @@ class YCellular(YFunction):
         # // max 2 minutes (each iteration may take up to 5 seconds if waiting)
         waitMore = 24
         while waitMore > 0:
-            #
             buff = self._download(cmd)
             bufflen = len(buff)
             buffstr = YByte2String(buff)
@@ -631,13 +836,13 @@ class YCellular(YFunction):
             while (idx > 0) and (YGetByte(buff, idx) != 64) and (YGetByte(buff, idx) != 10) and (YGetByte(buff, idx) != 13):
                 idx = idx - 1
             if YGetByte(buff, idx) == 64:
-                #
+                # // continuation detected
                 suffixlen = bufflen - idx
                 cmd = "at.txt?cmd=" + (buffstr)[buffstrlen - suffixlen: buffstrlen - suffixlen + suffixlen]
                 buffstr = (buffstr)[0: 0 + buffstrlen - suffixlen]
                 waitMore = waitMore - 1
             else:
-                #
+                # // request complete
                 waitMore = 0
             res = "" + res + "" + buffstr
         return res
@@ -656,7 +861,7 @@ class YCellular(YFunction):
         # idx
         # slen
         res = []
-        # // may throw an exception
+
         cops = self._AT("+COPS=?")
         slen = len(cops)
         del res[:]
@@ -672,14 +877,14 @@ class YCellular(YFunction):
                 if idx > 0:
                     res.append((cops)[0: 0 + idx])
             idx = cops.find("(")
-        
+
         return res
 
     def quickCellSurvey(self):
         """
         Returns a list of nearby cellular antennas, as required for quick
         geolocation of the device. The first cell listed is the serving
-        cell, and the next ones are the neighboor cells reported by the
+        cell, and the next ones are the neighbor cells reported by the
         serving cell.
 
         @return a list of YCellRecords.
@@ -699,7 +904,7 @@ class YCellular(YFunction):
         # tad
         # oper
         res = []
-        # // may throw an exception
+
         moni = self._AT("+CCED=0;#MONI=7;#MONI")
         mccs = (moni)[7: 7 + 3]
         if (mccs)[0: 0 + 1] == "0":
@@ -742,6 +947,9 @@ class YCellular(YFunction):
     def nextCellular(self):
         """
         Continues the enumeration of cellular interfaces started using yFirstCellular().
+        Caution: You can't make any assumption about the returned cellular interfaces order.
+        If you want to find a specific a cellular interface, use Cellular.findCellular()
+        and a hardwareID or a logical name.
 
         @return a pointer to a YCellular object, corresponding to
                 a cellular interface currently online, or a None pointer
@@ -756,7 +964,7 @@ class YCellular(YFunction):
 
 #--- (end of generated code: YCellular implementation)
 
-#--- (generated code: Cellular functions)
+#--- (generated code: YCellular functions)
 
     @staticmethod
     def FirstCellular():
@@ -790,4 +998,4 @@ class YCellular(YFunction):
 
         return YCellular.FindCellular(serialRef.value + "." + funcIdRef.value)
 
-#--- (end of generated code: Cellular functions)
+#--- (end of generated code: YCellular functions)
