@@ -1,35 +1,36 @@
+# -*- coding: utf-8 -*-
 #*********************************************************************
 #*
-#* $Id: yocto_gyro.py 22360 2015-12-15 13:31:40Z seb $
+#* $Id: yocto_gyro.py 38899 2019-12-20 17:21:03Z mvuilleu $
 #*
 #* Implements yFindGyro(), the high-level API for Gyro functions
 #*
-#* - - - - - - - - - License information: - - - - - - - - - 
+#* - - - - - - - - - License information: - - - - - - - - -
 #*
 #*  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
 #*
 #*  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
 #*  non-exclusive license to use, modify, copy and integrate this
-#*  file into your software for the sole purpose of interfacing 
-#*  with Yoctopuce products. 
+#*  file into your software for the sole purpose of interfacing
+#*  with Yoctopuce products.
 #*
-#*  You may reproduce and distribute copies of this file in 
+#*  You may reproduce and distribute copies of this file in
 #*  source or object form, as long as the sole purpose of this
-#*  code is to interface with Yoctopuce products. You must retain 
+#*  code is to interface with Yoctopuce products. You must retain
 #*  this notice in the distributed source file.
 #*
 #*  You should refer to Yoctopuce General Terms and Conditions
-#*  for additional information regarding your rights and 
+#*  for additional information regarding your rights and
 #*  obligations.
 #*
 #*  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
-#*  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING 
-#*  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS 
+#*  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
+#*  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
 #*  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
 #*  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
-#*  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, 
-#*  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR 
-#*  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT 
+#*  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
+#*  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
+#*  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
 #*  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
 #*  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
 #*  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
@@ -46,12 +47,15 @@ from yoctopuce.yocto_api import *
 #noinspection PyProtectedMember
 class YQt(YSensor):
     """
-    The Yoctopuce API YQt class provides direct access to the Yocto3D attitude estimation
-    using a quaternion. It is usually not needed to use the YQt class directly, as the
-    YGyro class provides a more convenient higher-level interface.
+    The YQt class provides direct access to the 3D attitude estimation
+    provided by Yoctopuce inertial sensors. The four instances of YQt
+    provide direct access to the individual quaternion components representing the
+    orientation. It is usually not needed to use the YQt class
+    directly, as the YGyro class provides a more convenient higher-level
+    interface.
 
     """
-#--- (end of generated code: YQt class start)
+    #--- (end of generated code: YQt class start)
     #--- (generated code: YQt return codes)
     #--- (end of generated code: YQt return codes)
     #--- (generated code: YQt definitions)
@@ -65,8 +69,8 @@ class YQt(YSensor):
         #--- (end of generated code: YQt attributes)
 
     #--- (generated code: YQt implementation)
-    def _parseAttr(self, member):
-        super(YQt, self)._parseAttr(member)
+    def _parseAttr(self, json_val):
+        super(YQt, self)._parseAttr(json_val)
 
     @staticmethod
     def FindQt(func):
@@ -89,7 +93,12 @@ class YQt(YSensor):
         found is returned. The search is performed first by hardware name,
         then by logical name.
 
-        @param func : a string that uniquely characterizes the quaternion component
+        If a call to this object's is_online() method returns FALSE although
+        you are certain that the matching device is plugged, make sure that you did
+        call registerHub() at application initialization time.
+
+        @param func : a string that uniquely characterizes the quaternion component, for instance
+                Y3DMK002.qt1.
 
         @return a YQt object allowing you to drive the quaternion component.
         """
@@ -103,6 +112,9 @@ class YQt(YSensor):
     def nextQt(self):
         """
         Continues the enumeration of quaternion components started using yFirstQt().
+        Caution: You can't make any assumption about the returned quaternion components order.
+        If you want to find a specific a quaternion component, use Qt.findQt()
+        and a hardwareID or a logical name.
 
         @return a pointer to a YQt object, corresponding to
                 a quaternion component currently online, or a None pointer
@@ -117,7 +129,7 @@ class YQt(YSensor):
 
 #--- (end of generated code: YQt implementation)
 
-#--- (generated code: Qt functions)
+#--- (generated code: YQt functions)
 
     @staticmethod
     def FirstQt():
@@ -151,7 +163,7 @@ class YQt(YSensor):
 
         return YQt.FindQt(serialRef.value + "." + funcIdRef.value)
 
-#--- (end of generated code: Qt functions)
+#--- (end of generated code: YQt functions)
 
 def yInternalGyroCallback(YQt_obj, str_value):
     gyro = YQt_obj.get_userData()
@@ -167,21 +179,19 @@ def yInternalGyroCallback(YQt_obj, str_value):
 #noinspection PyProtectedMember
 class YGyro(YSensor):
     """
-    The YSensor class is the parent class for all Yoctopuce sensors. It can be
-    used to read the current value and unit of any sensor, read the min/max
-    value, configure autonomous recording frequency and access recorded data.
-    It also provide a function to register a callback invoked each time the
-    observed value changes, or at a predefined interval. Using this class rather
-    than a specific subclass makes it possible to create generic applications
-    that work with any Yoctopuce sensor, even those that do not yet exist.
-    Note: The YAnButton class is the only analog input which does not inherit
-    from YSensor.
+    The YGyro class allows you to read and configure Yoctopuce gyroscopes.
+    It inherits from YSensor class the core functions to read measurements,
+    to register callback functions, and to access the autonomous datalogger.
+    This class adds the possibility to access x, y and z components of the rotation
+    vector separately, as well as the possibility to deal with quaternion-based
+    orientation estimates.
 
     """
-#--- (end of generated code: YGyro class start)
+    #--- (end of generated code: YGyro class start)
     #--- (generated code: YGyro return codes)
     #--- (end of generated code: YGyro return codes)
     #--- (generated code: YGyro definitions)
+    BANDWIDTH_INVALID = YAPI.INVALID_UINT
     XVALUE_INVALID = YAPI.INVALID_DOUBLE
     YVALUE_INVALID = YAPI.INVALID_DOUBLE
     ZVALUE_INVALID = YAPI.INVALID_DOUBLE
@@ -192,6 +202,7 @@ class YGyro(YSensor):
         self._className = 'Gyro'
         #--- (generated code: YGyro attributes)
         self._callback = None
+        self._bandwidth = YGyro.BANDWIDTH_INVALID
         self._xValue = YGyro.XVALUE_INVALID
         self._yValue = YGyro.YVALUE_INVALID
         self._zValue = YGyro.ZVALUE_INVALID
@@ -213,17 +224,47 @@ class YGyro(YSensor):
         #--- (end of generated code: YGyro attributes)
 
     #--- (generated code: YGyro implementation)
-    def _parseAttr(self, member):
-        if member.name == "xValue":
-            self._xValue = round(member.ivalue * 1000.0 / 65536.0) / 1000.0
-            return 1
-        if member.name == "yValue":
-            self._yValue = round(member.ivalue * 1000.0 / 65536.0) / 1000.0
-            return 1
-        if member.name == "zValue":
-            self._zValue = round(member.ivalue * 1000.0 / 65536.0) / 1000.0
-            return 1
-        super(YGyro, self)._parseAttr(member)
+    def _parseAttr(self, json_val):
+        if json_val.has("bandwidth"):
+            self._bandwidth = json_val.getInt("bandwidth")
+        if json_val.has("xValue"):
+            self._xValue = round(json_val.getDouble("xValue") * 1000.0 / 65536.0) / 1000.0
+        if json_val.has("yValue"):
+            self._yValue = round(json_val.getDouble("yValue") * 1000.0 / 65536.0) / 1000.0
+        if json_val.has("zValue"):
+            self._zValue = round(json_val.getDouble("zValue") * 1000.0 / 65536.0) / 1000.0
+        super(YGyro, self)._parseAttr(json_val)
+
+    def get_bandwidth(self):
+        """
+        Returns the measure update frequency, measured in Hz (Yocto-3D-V2 only).
+
+        @return an integer corresponding to the measure update frequency, measured in Hz (Yocto-3D-V2 only)
+
+        On failure, throws an exception or returns YGyro.BANDWIDTH_INVALID.
+        """
+        # res
+        if self._cacheExpiration <= YAPI.GetTickCount():
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
+                return YGyro.BANDWIDTH_INVALID
+        res = self._bandwidth
+        return res
+
+    def set_bandwidth(self, newval):
+        """
+        Changes the measure update frequency, measured in Hz (Yocto-3D-V2 only). When the
+        frequency is lower, the device performs averaging.
+        Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
+
+        @param newval : an integer corresponding to the measure update frequency, measured in Hz (Yocto-3D-V2 only)
+
+        @return YAPI.SUCCESS if the call succeeds.
+
+        On failure, throws an exception or returns a negative error code.
+        """
+        rest_val = str(newval)
+        return self._setAttr("bandwidth", rest_val)
 
     def get_xValue(self):
         """
@@ -234,10 +275,12 @@ class YGyro(YSensor):
 
         On failure, throws an exception or returns YGyro.XVALUE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGyro.XVALUE_INVALID
-        return self._xValue
+        res = self._xValue
+        return res
 
     def get_yValue(self):
         """
@@ -248,10 +291,12 @@ class YGyro(YSensor):
 
         On failure, throws an exception or returns YGyro.YVALUE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGyro.YVALUE_INVALID
-        return self._yValue
+        res = self._yValue
+        return res
 
     def get_zValue(self):
         """
@@ -262,10 +307,12 @@ class YGyro(YSensor):
 
         On failure, throws an exception or returns YGyro.ZVALUE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGyro.ZVALUE_INVALID
-        return self._zValue
+        res = self._zValue
+        return res
 
     @staticmethod
     def FindGyro(func):
@@ -288,7 +335,12 @@ class YGyro(YSensor):
         found is returned. The search is performed first by hardware name,
         then by logical name.
 
-        @param func : a string that uniquely characterizes the gyroscope
+        If a call to this object's is_online() method returns FALSE although
+        you are certain that the matching device is plugged, make sure that you did
+        call registerHub() at application initialization time.
+
+        @param func : a string that uniquely characterizes the gyroscope, for instance
+                Y3DMK002.gyro.
 
         @return a YGyro object allowing you to drive the gyroscope.
         """
@@ -302,7 +354,7 @@ class YGyro(YSensor):
     def _loadQuaternion(self):
         # now_stamp
         # age_ms
-        now_stamp = (YRelTickCount(YAPI.GetTickCount()) & (0x7FFFFFFF))
+        now_stamp = int((YRelTickCount(YAPI.GetTickCount()) & (0x7FFFFFFF)))
         age_ms = (((now_stamp - self._qt_stamp)) & (0x7FFFFFFF))
         if (age_ms >= 10) or (self._qt_stamp == 0):
             if self.load(10) != YAPI.SUCCESS:
@@ -312,7 +364,6 @@ class YGyro(YSensor):
                 self._qt_x = YQt.FindQt("" + self._serial + ".qt2")
                 self._qt_y = YQt.FindQt("" + self._serial + ".qt3")
                 self._qt_z = YQt.FindQt("" + self._serial + ".qt4")
-            #
             if self._qt_w.load(9) != YAPI.SUCCESS:
                 return YAPI.DEVICE_NOT_FOUND
             if self._qt_x.load(9) != YAPI.SUCCESS:
@@ -335,7 +386,7 @@ class YGyro(YSensor):
         # sqz
         # norm
         # delta
-        # // may throw an exception
+
         if self._loadQuaternion() != YAPI.SUCCESS:
             return YAPI.DEVICE_NOT_FOUND
         if self._angles_stamp != self._qt_stamp:
@@ -346,14 +397,14 @@ class YGyro(YSensor):
             norm = sqx + sqy + sqz + sqw
             delta = self._y * self._w - self._x * self._z
             if delta > 0.499 * norm:
-                #
+                # // singularity at north pole
                 self._pitch = 90.0
-                self._head  = round(2.0 * 1800.0/math.pi * math.atan2(self._x,self._w)) / 10.0
+                self._head  = round(2.0 * 1800.0/math.pi * math.atan2(self._x,-self._w)) / 10.0
             else:
                 if delta < -0.499 * norm:
-                    #
+                    # // singularity at south pole
                     self._pitch = -90.0
-                    self._head  = round(-2.0 * 1800.0/math.pi * math.atan2(self._x,self._w)) / 10.0
+                    self._head  = round(-2.0 * 1800.0/math.pi * math.atan2(self._x,-self._w)) / 10.0
                 else:
                     self._roll  = round(1800.0/math.pi * math.atan2(2.0 * (self._w * self._x + self._y * self._z),sqw - sqx - sqy + sqz)) / 10.0
                     self._pitch = round(1800.0/math.pi * math.asin(2.0 * delta / norm)) / 10.0
@@ -373,7 +424,6 @@ class YGyro(YSensor):
         @return a floating-point number corresponding to roll angle
                 in degrees, between -180 and +180.
         """
-        # // may throw an exception
         self._loadAngles()
         return self._roll
 
@@ -389,7 +439,6 @@ class YGyro(YSensor):
         @return a floating-point number corresponding to pitch angle
                 in degrees, between -90 and +90.
         """
-        # // may throw an exception
         self._loadAngles()
         return self._pitch
 
@@ -405,7 +454,6 @@ class YGyro(YSensor):
         @return a floating-point number corresponding to heading
                 in degrees, between 0 and 360.
         """
-        # // may throw an exception
         self._loadAngles()
         return self._head
 
@@ -419,7 +467,6 @@ class YGyro(YSensor):
         @return a floating-point number corresponding to the w
                 component of the quaternion.
         """
-        # // may throw an exception
         self._loadQuaternion()
         return self._w
 
@@ -434,7 +481,6 @@ class YGyro(YSensor):
         @return a floating-point number corresponding to the x
                 component of the quaternion.
         """
-        # // may throw an exception
         self._loadQuaternion()
         return self._x
 
@@ -449,7 +495,6 @@ class YGyro(YSensor):
         @return a floating-point number corresponding to the y
                 component of the quaternion.
         """
-        # // may throw an exception
         self._loadQuaternion()
         return self._y
 
@@ -464,7 +509,6 @@ class YGyro(YSensor):
         @return a floating-point number corresponding to the z
                 component of the quaternion.
         """
-        # // may throw an exception
         self._loadQuaternion()
         return self._z
 
@@ -486,7 +530,6 @@ class YGyro(YSensor):
         """
         self._quatCallback = callback
         if callback is not None:
-            #
             if self._loadQuaternion() != YAPI.SUCCESS:
                 return YAPI.DEVICE_NOT_FOUND
             self._qt_w.set_userData(self)
@@ -523,7 +566,6 @@ class YGyro(YSensor):
         """
         self._anglesCallback = callback
         if callback is not None:
-            #
             if self._loadQuaternion() != YAPI.SUCCESS:
                 return YAPI.DEVICE_NOT_FOUND
             self._qt_w.set_userData(self)
@@ -553,11 +595,10 @@ class YGyro(YSensor):
             self._z = qtValue
         if qtIndex < 4:
             return 0
-        self._qt_stamp = (YRelTickCount(YAPI.GetTickCount()) & (0x7FFFFFFF))
+        self._qt_stamp = int((YRelTickCount(YAPI.GetTickCount()) & (0x7FFFFFFF)))
         if self._quatCallback is not None:
             self._quatCallback(self, self._w, self._x, self._y, self._z)
         if self._anglesCallback is not None:
-            #
             self._loadAngles()
             self._anglesCallback(self, self._roll, self._pitch, self._head)
         return 0
@@ -565,6 +606,9 @@ class YGyro(YSensor):
     def nextGyro(self):
         """
         Continues the enumeration of gyroscopes started using yFirstGyro().
+        Caution: You can't make any assumption about the returned gyroscopes order.
+        If you want to find a specific a gyroscope, use Gyro.findGyro()
+        and a hardwareID or a logical name.
 
         @return a pointer to a YGyro object, corresponding to
                 a gyroscope currently online, or a None pointer
@@ -579,7 +623,7 @@ class YGyro(YSensor):
 
 #--- (end of generated code: YGyro implementation)
 
-#--- (generated code: Gyro functions)
+#--- (generated code: YGyro functions)
 
     @staticmethod
     def FirstGyro():
@@ -613,4 +657,4 @@ class YGyro(YSensor):
 
         return YGyro.FindGyro(serialRef.value + "." + funcIdRef.value)
 
-#--- (end of generated code: Gyro functions)
+#--- (end of generated code: YGyro functions)
