@@ -1,44 +1,46 @@
-#*********************************************************************
-#*
-#* $Id: yocto_temperature.py 23527 2016-03-18 21:49:19Z mvuilleu $
-#*
-#* Implements yFindTemperature(), the high-level API for Temperature functions
-#*
-#* - - - - - - - - - License information: - - - - - - - - - 
-#*
-#*  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
-#*
-#*  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
-#*  non-exclusive license to use, modify, copy and integrate this
-#*  file into your software for the sole purpose of interfacing
-#*  with Yoctopuce products.
-#*
-#*  You may reproduce and distribute copies of this file in
-#*  source or object form, as long as the sole purpose of this
-#*  code is to interface with Yoctopuce products. You must retain
-#*  this notice in the distributed source file.
-#*
-#*  You should refer to Yoctopuce General Terms and Conditions
-#*  for additional information regarding your rights and
-#*  obligations.
-#*
-#*  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
-#*  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING 
-#*  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
-#*  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
-#*  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
-#*  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
-#*  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
-#*  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
-#*  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
-#*  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
-#*  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
-#*  WARRANTY, OR OTHERWISE.
-#*
-#*********************************************************************/
+# -*- coding: utf-8 -*-
+# ********************************************************************
+#
+#  $Id: yocto_temperature.py 39648 2020-03-12 13:56:10Z mvuilleu $
+#
+#  Implements yFindTemperature(), the high-level API for Temperature functions
+#
+#  - - - - - - - - - License information: - - - - - - - - -
+#
+#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
+#
+#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
+#  non-exclusive license to use, modify, copy and integrate this
+#  file into your software for the sole purpose of interfacing
+#  with Yoctopuce products.
+#
+#  You may reproduce and distribute copies of this file in
+#  source or object form, as long as the sole purpose of this
+#  code is to interface with Yoctopuce products. You must retain
+#  this notice in the distributed source file.
+#
+#  You should refer to Yoctopuce General Terms and Conditions
+#  for additional information regarding your rights and
+#  obligations.
+#
+#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
+#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
+#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
+#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
+#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
+#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
+#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
+#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
+#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
+#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
+#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
+#  WARRANTY, OR OTHERWISE.
+#
+# *********************************************************************
 
 
 __docformat__ = 'restructuredtext en'
+import math
 from yoctopuce.yocto_api import *
 
 
@@ -46,18 +48,20 @@ from yoctopuce.yocto_api import *
 #noinspection PyProtectedMember
 class YTemperature(YSensor):
     """
-    The Yoctopuce class YTemperature allows you to read and configure Yoctopuce temperature
-    sensors. It inherits from YSensor class the core functions to read measurements,
-    register callback functions, access to the autonomous datalogger.
-    This class adds the ability to configure some specific parameters for some
-    sensors (connection type, temperature mapping table).
+    The YTemperature class allows you to read and configure Yoctopuce temperature sensors.
+    It inherits from YSensor class the core functions to read measurements,
+    to register callback functions, and to access the autonomous datalogger.
+    This class adds the ability to configure some specific parameters
+    for some sensors (connection type, temperature mapping table).
 
     """
-#--- (end of YTemperature class start)
+    #--- (end of YTemperature class start)
     #--- (YTemperature return codes)
     #--- (end of YTemperature return codes)
     #--- (YTemperature dlldef)
     #--- (end of YTemperature dlldef)
+    #--- (YTemperature yapiwrapper)
+    #--- (end of YTemperature yapiwrapper)
     #--- (YTemperature definitions)
     SIGNALVALUE_INVALID = YAPI.INVALID_DOUBLE
     SIGNALUNIT_INVALID = YAPI.INVALID_STRING
@@ -76,6 +80,10 @@ class YTemperature(YSensor):
     SENSORTYPE_RES_OHM = 11
     SENSORTYPE_RES_NTC = 12
     SENSORTYPE_RES_LINEAR = 13
+    SENSORTYPE_RES_INTERNAL = 14
+    SENSORTYPE_IR = 15
+    SENSORTYPE_RES_PT1000 = 16
+    SENSORTYPE_CHANNEL_OFF = 17
     SENSORTYPE_INVALID = -1
     #--- (end of YTemperature definitions)
 
@@ -91,20 +99,16 @@ class YTemperature(YSensor):
         #--- (end of YTemperature attributes)
 
     #--- (YTemperature implementation)
-    def _parseAttr(self, member):
-        if member.name == "sensorType":
-            self._sensorType = member.ivalue
-            return 1
-        if member.name == "signalValue":
-            self._signalValue = round(member.ivalue * 1000.0 / 65536.0) / 1000.0
-            return 1
-        if member.name == "signalUnit":
-            self._signalUnit = member.svalue
-            return 1
-        if member.name == "command":
-            self._command = member.svalue
-            return 1
-        super(YTemperature, self)._parseAttr(member)
+    def _parseAttr(self, json_val):
+        if json_val.has("sensorType"):
+            self._sensorType = json_val.getInt("sensorType")
+        if json_val.has("signalValue"):
+            self._signalValue = round(json_val.getDouble("signalValue") * 1000.0 / 65536.0) / 1000.0
+        if json_val.has("signalUnit"):
+            self._signalUnit = json_val.getString("signalUnit")
+        if json_val.has("command"):
+            self._command = json_val.getString("command")
+        super(YTemperature, self)._parseAttr(json_val)
 
     def set_unit(self, newval):
         """
@@ -136,19 +140,23 @@ class YTemperature(YSensor):
         YTemperature.SENSORTYPE_TYPE_R, YTemperature.SENSORTYPE_TYPE_S, YTemperature.SENSORTYPE_TYPE_T,
         YTemperature.SENSORTYPE_PT100_4WIRES, YTemperature.SENSORTYPE_PT100_3WIRES,
         YTemperature.SENSORTYPE_PT100_2WIRES, YTemperature.SENSORTYPE_RES_OHM,
-        YTemperature.SENSORTYPE_RES_NTC and YTemperature.SENSORTYPE_RES_LINEAR corresponding to the
+        YTemperature.SENSORTYPE_RES_NTC, YTemperature.SENSORTYPE_RES_LINEAR,
+        YTemperature.SENSORTYPE_RES_INTERNAL, YTemperature.SENSORTYPE_IR,
+        YTemperature.SENSORTYPE_RES_PT1000 and YTemperature.SENSORTYPE_CHANNEL_OFF corresponding to the
         temperature sensor type
 
         On failure, throws an exception or returns YTemperature.SENSORTYPE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YTemperature.SENSORTYPE_INVALID
-        return self._sensorType
+        res = self._sensorType
+        return res
 
     def set_sensorType(self, newval):
         """
-        Modifies the temperature sensor type.  This function is used
+        Changes the temperature sensor type.  This function is used
         to define the type of thermocouple (K,E...) used with the device.
         It has no effect if module is using a digital sensor or a thermistor.
         Remember to call the saveToFlash() method of the module if the
@@ -159,7 +167,10 @@ class YTemperature(YSensor):
         YTemperature.SENSORTYPE_TYPE_R, YTemperature.SENSORTYPE_TYPE_S, YTemperature.SENSORTYPE_TYPE_T,
         YTemperature.SENSORTYPE_PT100_4WIRES, YTemperature.SENSORTYPE_PT100_3WIRES,
         YTemperature.SENSORTYPE_PT100_2WIRES, YTemperature.SENSORTYPE_RES_OHM,
-        YTemperature.SENSORTYPE_RES_NTC and YTemperature.SENSORTYPE_RES_LINEAR
+        YTemperature.SENSORTYPE_RES_NTC, YTemperature.SENSORTYPE_RES_LINEAR,
+        YTemperature.SENSORTYPE_RES_INTERNAL, YTemperature.SENSORTYPE_IR,
+        YTemperature.SENSORTYPE_RES_PT1000 and YTemperature.SENSORTYPE_CHANNEL_OFF corresponding to the
+        temperature sensor type
 
         @return YAPI.SUCCESS if the call succeeds.
 
@@ -177,10 +188,12 @@ class YTemperature(YSensor):
 
         On failure, throws an exception or returns YTemperature.SIGNALVALUE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YTemperature.SIGNALVALUE_INVALID
-        return round(self._signalValue * 1000) / 1000
+        res = round(self._signalValue * 1000) / 1000
+        return res
 
     def get_signalUnit(self):
         """
@@ -190,16 +203,20 @@ class YTemperature(YSensor):
 
         On failure, throws an exception or returns YTemperature.SIGNALUNIT_INVALID.
         """
-        if self._cacheExpiration == datetime.datetime.fromtimestamp(0):
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+        # res
+        if self._cacheExpiration == datetime.datetime.fromtimestamp(86400):
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YTemperature.SIGNALUNIT_INVALID
-        return self._signalUnit
+        res = self._signalUnit
+        return res
 
     def get_command(self):
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YTemperature.COMMAND_INVALID
-        return self._command
+        res = self._command
+        return res
 
     def set_command(self, newval):
         rest_val = newval
@@ -226,7 +243,12 @@ class YTemperature(YSensor):
         found is returned. The search is performed first by hardware name,
         then by logical name.
 
-        @param func : a string that uniquely characterizes the temperature sensor
+        If a call to this object's is_online() method returns FALSE although
+        you are certain that the matching device is plugged, make sure that you did
+        call registerHub() at application initialization time.
+
+        @param func : a string that uniquely characterizes the temperature sensor, for instance
+                METEOMK2.temperature.
 
         @return a YTemperature object allowing you to drive the temperature sensor.
         """
@@ -239,7 +261,7 @@ class YTemperature(YSensor):
 
     def set_ntcParameters(self, res25, beta):
         """
-        Configure NTC thermistor parameters in order to properly compute the temperature from
+        Configures NTC thermistor parameters in order to properly compute the temperature from
         the measured resistance. For increased precision, you can enter a complete mapping
         table using set_thermistorResponseTable. This function can only be used with a
         temperature sensor based on thermistors.
@@ -258,15 +280,15 @@ class YTemperature(YSensor):
         resValues = []
         t0 = 25.0+275.15
         t1 = 100.0+275.15
-        res100 = res25 * exp(beta*(1.0/t1 - 1.0/t0))
+        res100 = res25 * math.exp(beta*(1.0/t1 - 1.0/t0))
         del tempValues[:]
         del resValues[:]
         tempValues.append(25.0)
         resValues.append(res25)
         tempValues.append(100.0)
         resValues.append(res100)
-        
-        
+
+
         return self.set_thermistorResponseTable(tempValues, resValues)
 
     def set_thermistorResponseTable(self, tempValues, resValues):
@@ -276,7 +298,7 @@ class YTemperature(YSensor):
         sensor based on thermistors.
 
         @param tempValues : array of floating point numbers, corresponding to all
-                temperatures (in degrees Celcius) for which the resistance of the
+                temperatures (in degrees Celsius) for which the resistance of the
                 thermistor is specified.
         @param resValues : array of floating point numbers, corresponding to the resistance
                 values (in Ohms) for each of the temperature included in the first
@@ -297,12 +319,15 @@ class YTemperature(YSensor):
         siz = len(tempValues)
         if not (siz >= 2):
             self._throw(YAPI.INVALID_ARGUMENT, "thermistor response table must have at least two points")
+            return YAPI.INVALID_ARGUMENT
         if not (siz == len(resValues)):
             self._throw(YAPI.INVALID_ARGUMENT, "table sizes mismatch")
-        # // may throw an exception
+            return YAPI.INVALID_ARGUMENT
+
         res = self.set_command("Z")
         if not (res==YAPI.SUCCESS):
             self._throw(YAPI.IO_ERROR, "unable to reset thermistor parameters")
+            return YAPI.IO_ERROR
         # // add records in growing resistance value
         found = 1
         prev = 0.0
@@ -322,6 +347,7 @@ class YTemperature(YSensor):
                 res = self.set_command("m" + str(int(round(1000*curr))) + ":" + str(int(round(1000*currTemp))))
                 if not (res==YAPI.SUCCESS):
                     self._throw(YAPI.IO_ERROR, "unable to reset thermistor parameters")
+                    return YAPI.IO_ERROR
                 prev = curr
         return YAPI.SUCCESS
 
@@ -332,7 +358,7 @@ class YTemperature(YSensor):
         temperature sensor based on thermistors.
 
         @param tempValues : array of floating point numbers, that is filled by the function
-                with all temperatures (in degrees Celcius) for which the resistance
+                with all temperatures (in degrees Celsius) for which the resistance
                 of the thermistor is specified.
         @param resValues : array of floating point numbers, that is filled by the function
                 with the value (in Ohms) for each of the temperature included in the
@@ -355,9 +381,11 @@ class YTemperature(YSensor):
         # currRes
         del tempValues[:]
         del resValues[:]
-        # // may throw an exception
+
         id = self.get_functionId()
         id = (id)[11: 11 + len(id) - 11]
+        if id == "":
+            id = "1"
         bin_json = self._download("extra.json?page=" + id)
         paramlist = self._json_get_array(bin_json)
         # // first convert all temperatures to float
@@ -389,13 +417,16 @@ class YTemperature(YSensor):
                 tempValues.append(curr)
                 resValues.append(currRes)
                 prev = curr
-        
-        
+
+
         return YAPI.SUCCESS
 
     def nextTemperature(self):
         """
         Continues the enumeration of temperature sensors started using yFirstTemperature().
+        Caution: You can't make any assumption about the returned temperature sensors order.
+        If you want to find a specific a temperature sensor, use Temperature.findTemperature()
+        and a hardwareID or a logical name.
 
         @return a pointer to a YTemperature object, corresponding to
                 a temperature sensor currently online, or a None pointer
@@ -410,7 +441,7 @@ class YTemperature(YSensor):
 
 #--- (end of YTemperature implementation)
 
-#--- (Temperature functions)
+#--- (YTemperature functions)
 
     @staticmethod
     def FirstTemperature():
@@ -444,4 +475,4 @@ class YTemperature(YSensor):
 
         return YTemperature.FindTemperature(serialRef.value + "." + funcIdRef.value)
 
-#--- (end of Temperature functions)
+#--- (end of YTemperature functions)
