@@ -1,41 +1,42 @@
-#*********************************************************************
-#*
-#* $Id: yocto_watchdog.py 23243 2016-02-23 14:13:12Z seb $
-#*
-#* Implements yFindWatchdog(), the high-level API for Watchdog functions
-#*
-#* - - - - - - - - - License information: - - - - - - - - - 
-#*
-#*  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
-#*
-#*  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
-#*  non-exclusive license to use, modify, copy and integrate this
-#*  file into your software for the sole purpose of interfacing
-#*  with Yoctopuce products.
-#*
-#*  You may reproduce and distribute copies of this file in
-#*  source or object form, as long as the sole purpose of this
-#*  code is to interface with Yoctopuce products. You must retain
-#*  this notice in the distributed source file.
-#*
-#*  You should refer to Yoctopuce General Terms and Conditions
-#*  for additional information regarding your rights and
-#*  obligations.
-#*
-#*  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
-#*  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING 
-#*  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
-#*  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
-#*  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
-#*  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
-#*  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
-#*  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
-#*  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
-#*  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
-#*  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
-#*  WARRANTY, OR OTHERWISE.
-#*
-#*********************************************************************/
+# -*- coding: utf-8 -*-
+# ********************************************************************
+#
+#  $Id: yocto_watchdog.py 38899 2019-12-20 17:21:03Z mvuilleu $
+#
+#  Implements yFindWatchdog(), the high-level API for Watchdog functions
+#
+#  - - - - - - - - - License information: - - - - - - - - -
+#
+#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
+#
+#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
+#  non-exclusive license to use, modify, copy and integrate this
+#  file into your software for the sole purpose of interfacing
+#  with Yoctopuce products.
+#
+#  You may reproduce and distribute copies of this file in
+#  source or object form, as long as the sole purpose of this
+#  code is to interface with Yoctopuce products. You must retain
+#  this notice in the distributed source file.
+#
+#  You should refer to Yoctopuce General Terms and Conditions
+#  for additional information regarding your rights and
+#  obligations.
+#
+#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
+#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
+#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
+#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
+#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
+#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
+#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
+#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
+#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
+#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
+#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
+#  WARRANTY, OR OTHERWISE.
+#
+# *********************************************************************
 
 
 __docformat__ = 'restructuredtext en'
@@ -46,19 +47,23 @@ from yoctopuce.yocto_api import *
 #noinspection PyProtectedMember
 class YWatchdog(YFunction):
     """
-    The watchog function works like a relay and can cause a brief power cut
-    to an appliance after a preset delay to force this appliance to
-    reset. The Watchdog must be called from time to time to reset the
-    timer and prevent the appliance reset.
-    The watchdog can be driven direcly with <i>pulse</i> and <i>delayedpulse</i> methods to switch
-    off an appliance for a given duration.
+    The YWatchdog class allows you to drive a Yoctopuce watchdog.
+    A watchdog works like a relay, with an extra timer that can automatically
+    trigger a brief power cycle to an appliance after a preset delay, to force this
+    appliance to reset if a problem occurs. During normal use, the watchdog timer
+    is reset periodically by the application to prevent the automated power cycle.
+    Whenever the application dies, the watchdog will automatically trigger the power cycle.
+    The watchdog can also be driven directly with pulse and delayedPulse
+    methods to switch off an appliance for a given duration.
 
     """
-#--- (end of YWatchdog class start)
+    #--- (end of YWatchdog class start)
     #--- (YWatchdog return codes)
     #--- (end of YWatchdog return codes)
     #--- (YWatchdog dlldef)
     #--- (end of YWatchdog dlldef)
+    #--- (YWatchdog yapiwrapper)
+    #--- (end of YWatchdog yapiwrapper)
     #--- (YWatchdog definitions)
     MAXTIMEONSTATEA_INVALID = YAPI.INVALID_LONG
     MAXTIMEONSTATEB_INVALID = YAPI.INVALID_LONG
@@ -102,56 +107,43 @@ class YWatchdog(YFunction):
         self._running = YWatchdog.RUNNING_INVALID
         self._triggerDelay = YWatchdog.TRIGGERDELAY_INVALID
         self._triggerDuration = YWatchdog.TRIGGERDURATION_INVALID
+        self._firm = 0
         #--- (end of YWatchdog attributes)
 
     #--- (YWatchdog implementation)
-    def _parseAttr(self, member):
-        if member.name == "state":
-            self._state = member.ivalue
-            return 1
-        if member.name == "stateAtPowerOn":
-            self._stateAtPowerOn = member.ivalue
-            return 1
-        if member.name == "maxTimeOnStateA":
-            self._maxTimeOnStateA = member.ivalue
-            return 1
-        if member.name == "maxTimeOnStateB":
-            self._maxTimeOnStateB = member.ivalue
-            return 1
-        if member.name == "output":
-            self._output = member.ivalue
-            return 1
-        if member.name == "pulseTimer":
-            self._pulseTimer = member.ivalue
-            return 1
-        if member.name == "delayedPulseTimer":
-            if member.recordtype != YAPI.TJSONRECORDTYPE.JSON_STRUCT:
-                self._delayedPulseTimer = -1
+    def _parseAttr(self, json_val):
+        if json_val.has("state"):
+            self._state = (json_val.getInt("state") > 0 if 1 else 0)
+        if json_val.has("stateAtPowerOn"):
+            self._stateAtPowerOn = json_val.getInt("stateAtPowerOn")
+        if json_val.has("maxTimeOnStateA"):
+            self._maxTimeOnStateA = json_val.getLong("maxTimeOnStateA")
+        if json_val.has("maxTimeOnStateB"):
+            self._maxTimeOnStateB = json_val.getLong("maxTimeOnStateB")
+        if json_val.has("output"):
+            self._output = (json_val.getInt("output") > 0 if 1 else 0)
+        if json_val.has("pulseTimer"):
+            self._pulseTimer = json_val.getLong("pulseTimer")
+        if json_val.has("delayedPulseTimer"):
+            subjson = json_val.getYJSONObject("delayedPulseTimer");
             self._delayedPulseTimer = {"moving": None, "target": None, "ms": None}
-            for submemb in member.members:
-                if submemb.name == "moving":
-                    self._delayedPulseTimer["moving"] = submemb.ivalue
-                elif submemb.name == "target":
-                    self._delayedPulseTimer["target"] = submemb.ivalue
-                elif submemb.name == "ms":
-                    self._delayedPulseTimer["ms"] = submemb.ivalue
-            return 1
-        if member.name == "countdown":
-            self._countdown = member.ivalue
-            return 1
-        if member.name == "autoStart":
-            self._autoStart = member.ivalue
-            return 1
-        if member.name == "running":
-            self._running = member.ivalue
-            return 1
-        if member.name == "triggerDelay":
-            self._triggerDelay = member.ivalue
-            return 1
-        if member.name == "triggerDuration":
-            self._triggerDuration = member.ivalue
-            return 1
-        super(YWatchdog, self)._parseAttr(member)
+            if subjson.has("moving"):
+                self._delayedPulseTimer["moving"] = subjson.getInt("moving")
+            if subjson.has("target"):
+                self._delayedPulseTimer["target"] = subjson.getInt("target")
+            if subjson.has("ms"):
+                self._delayedPulseTimer["ms"] = subjson.getInt("ms")
+        if json_val.has("countdown"):
+            self._countdown = json_val.getLong("countdown")
+        if json_val.has("autoStart"):
+            self._autoStart = (json_val.getInt("autoStart") > 0 if 1 else 0)
+        if json_val.has("running"):
+            self._running = (json_val.getInt("running") > 0 if 1 else 0)
+        if json_val.has("triggerDelay"):
+            self._triggerDelay = json_val.getLong("triggerDelay")
+        if json_val.has("triggerDuration"):
+            self._triggerDuration = json_val.getLong("triggerDuration")
+        super(YWatchdog, self)._parseAttr(json_val)
 
     def get_state(self):
         """
@@ -162,10 +154,12 @@ class YWatchdog(YFunction):
 
         On failure, throws an exception or returns YWatchdog.STATE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YWatchdog.STATE_INVALID
-        return self._state
+        res = self._state
+        return res
 
     def set_state(self, newval):
         """
@@ -183,28 +177,34 @@ class YWatchdog(YFunction):
 
     def get_stateAtPowerOn(self):
         """
-        Returns the state of the watchdog at device startup (A for the idle position, B for the active
-        position, UNCHANGED for no change).
+        Returns the state of the watchdog at device startup (A for the idle position,
+        B for the active position, UNCHANGED to leave the relay state as is).
 
         @return a value among YWatchdog.STATEATPOWERON_UNCHANGED, YWatchdog.STATEATPOWERON_A and
         YWatchdog.STATEATPOWERON_B corresponding to the state of the watchdog at device startup (A for the
-        idle position, B for the active position, UNCHANGED for no change)
+        idle position,
+                B for the active position, UNCHANGED to leave the relay state as is)
 
         On failure, throws an exception or returns YWatchdog.STATEATPOWERON_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YWatchdog.STATEATPOWERON_INVALID
-        return self._stateAtPowerOn
+        res = self._stateAtPowerOn
+        return res
 
     def set_stateAtPowerOn(self, newval):
         """
-        Preset the state of the watchdog at device startup (A for the idle position,
-        B for the active position, UNCHANGED for no modification). Remember to call the matching module saveToFlash()
+        Changes the state of the watchdog at device startup (A for the idle position,
+        B for the active position, UNCHANGED to leave the relay state as is).
+        Remember to call the matching module saveToFlash()
         method, otherwise this call will have no effect.
 
         @param newval : a value among YWatchdog.STATEATPOWERON_UNCHANGED, YWatchdog.STATEATPOWERON_A and
-        YWatchdog.STATEATPOWERON_B
+        YWatchdog.STATEATPOWERON_B corresponding to the state of the watchdog at device startup (A for the
+        idle position,
+                B for the active position, UNCHANGED to leave the relay state as is)
 
         @return YAPI.SUCCESS if the call succeeds.
 
@@ -215,24 +215,30 @@ class YWatchdog(YFunction):
 
     def get_maxTimeOnStateA(self):
         """
-        Retourne the maximum time (ms) allowed for $THEFUNCTIONS$ to stay in state A before automatically
-        switching back in to B state. Zero means no maximum time.
+        Returns the maximum time (ms) allowed for the watchdog to stay in state
+        A before automatically switching back in to B state. Zero means no time limit.
 
-        @return an integer
+        @return an integer corresponding to the maximum time (ms) allowed for the watchdog to stay in state
+                A before automatically switching back in to B state
 
         On failure, throws an exception or returns YWatchdog.MAXTIMEONSTATEA_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YWatchdog.MAXTIMEONSTATEA_INVALID
-        return self._maxTimeOnStateA
+        res = self._maxTimeOnStateA
+        return res
 
     def set_maxTimeOnStateA(self, newval):
         """
-        Sets the maximum time (ms) allowed for $THEFUNCTIONS$ to stay in state A before automatically
-        switching back in to B state. Use zero for no maximum time.
+        Changes the maximum time (ms) allowed for the watchdog to stay in state A
+        before automatically switching back in to B state. Use zero for no time limit.
+        Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
 
-        @param newval : an integer
+        @param newval : an integer corresponding to the maximum time (ms) allowed for the watchdog to stay in state A
+                before automatically switching back in to B state
 
         @return YAPI.SUCCESS if the call succeeds.
 
@@ -243,24 +249,30 @@ class YWatchdog(YFunction):
 
     def get_maxTimeOnStateB(self):
         """
-        Retourne the maximum time (ms) allowed for $THEFUNCTIONS$ to stay in state B before automatically
-        switching back in to A state. Zero means no maximum time.
+        Retourne the maximum time (ms) allowed for the watchdog to stay in state B
+        before automatically switching back in to A state. Zero means no time limit.
 
         @return an integer
 
         On failure, throws an exception or returns YWatchdog.MAXTIMEONSTATEB_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YWatchdog.MAXTIMEONSTATEB_INVALID
-        return self._maxTimeOnStateB
+        res = self._maxTimeOnStateB
+        return res
 
     def set_maxTimeOnStateB(self, newval):
         """
-        Sets the maximum time (ms) allowed for $THEFUNCTIONS$ to stay in state B before automatically
-        switching back in to A state. Use zero for no maximum time.
+        Changes the maximum time (ms) allowed for the watchdog to stay in state B before
+        automatically switching back in to A state. Use zero for no time limit.
+        Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
 
-        @param newval : an integer
+        @param newval : an integer corresponding to the maximum time (ms) allowed for the watchdog to stay
+        in state B before
+                automatically switching back in to A state
 
         @return YAPI.SUCCESS if the call succeeds.
 
@@ -278,10 +290,12 @@ class YWatchdog(YFunction):
 
         On failure, throws an exception or returns YWatchdog.OUTPUT_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YWatchdog.OUTPUT_INVALID
-        return self._output
+        res = self._output
+        return res
 
     def set_output(self, newval):
         """
@@ -308,10 +322,12 @@ class YWatchdog(YFunction):
 
         On failure, throws an exception or returns YWatchdog.PULSETIMER_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YWatchdog.PULSETIMER_INVALID
-        return self._pulseTimer
+        res = self._pulseTimer
+        return res
 
     def set_pulseTimer(self, newval):
         rest_val = str(newval)
@@ -322,7 +338,7 @@ class YWatchdog(YFunction):
         Sets the relay to output B (active) for a specified duration, then brings it
         automatically back to output A (idle state).
 
-        @param ms_duration : pulse duration, in millisecondes
+        @param ms_duration : pulse duration, in milliseconds
 
         @return YAPI.SUCCESS if the call succeeds.
 
@@ -332,10 +348,12 @@ class YWatchdog(YFunction):
         return self._setAttr("pulseTimer", rest_val)
 
     def get_delayedPulseTimer(self):
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YWatchdog.DELAYEDPULSETIMER_INVALID
-        return self._delayedPulseTimer
+        res = self._delayedPulseTimer
+        return res
 
     def set_delayedPulseTimer(self, newval):
         rest_val = str(newval.target) + ":" + str(newval.ms)
@@ -345,8 +363,8 @@ class YWatchdog(YFunction):
         """
         Schedules a pulse.
 
-        @param ms_delay : waiting time before the pulse, in millisecondes
-        @param ms_duration : pulse duration, in millisecondes
+        @param ms_delay : waiting time before the pulse, in milliseconds
+        @param ms_duration : pulse duration, in milliseconds
 
         @return YAPI.SUCCESS if the call succeeds.
 
@@ -365,32 +383,36 @@ class YWatchdog(YFunction):
 
         On failure, throws an exception or returns YWatchdog.COUNTDOWN_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YWatchdog.COUNTDOWN_INVALID
-        return self._countdown
+        res = self._countdown
+        return res
 
     def get_autoStart(self):
         """
-        Returns the watchdog runing state at module power on.
+        Returns the watchdog running state at module power on.
 
-        @return either YWatchdog.AUTOSTART_OFF or YWatchdog.AUTOSTART_ON, according to the watchdog runing
+        @return either YWatchdog.AUTOSTART_OFF or YWatchdog.AUTOSTART_ON, according to the watchdog running
         state at module power on
 
         On failure, throws an exception or returns YWatchdog.AUTOSTART_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YWatchdog.AUTOSTART_INVALID
-        return self._autoStart
+        res = self._autoStart
+        return res
 
     def set_autoStart(self, newval):
         """
-        Changes the watchdog runningsttae at module power on. Remember to call the
+        Changes the watchdog running state at module power on. Remember to call the
         saveToFlash() method and then to reboot the module to apply this setting.
 
         @param newval : either YWatchdog.AUTOSTART_OFF or YWatchdog.AUTOSTART_ON, according to the watchdog
-        runningsttae at module power on
+        running state at module power on
 
         @return YAPI.SUCCESS if the call succeeds.
 
@@ -407,10 +429,12 @@ class YWatchdog(YFunction):
 
         On failure, throws an exception or returns YWatchdog.RUNNING_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YWatchdog.RUNNING_INVALID
-        return self._running
+        res = self._running
+        return res
 
     def set_running(self, newval):
         """
@@ -429,7 +453,7 @@ class YWatchdog(YFunction):
     def resetWatchdog(self):
         """
         Resets the watchdog. When the watchdog is running, this function
-        must be called on a regular basis to prevent the watchog to
+        must be called on a regular basis to prevent the watchdog to
         trigger
 
         @return YAPI.SUCCESS if the call succeeds.
@@ -448,17 +472,21 @@ class YWatchdog(YFunction):
 
         On failure, throws an exception or returns YWatchdog.TRIGGERDELAY_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YWatchdog.TRIGGERDELAY_INVALID
-        return self._triggerDelay
+        res = self._triggerDelay
+        return res
 
     def set_triggerDelay(self, newval):
         """
-        Changes the waiting delay before a reset is triggered by the watchdog, in milliseconds.
+        Changes the waiting delay before a reset is triggered by the watchdog,
+        in milliseconds. Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
 
-        @param newval : an integer corresponding to the waiting delay before a reset is triggered by the
-        watchdog, in milliseconds
+        @param newval : an integer corresponding to the waiting delay before a reset is triggered by the watchdog,
+                in milliseconds
 
         @return YAPI.SUCCESS if the call succeeds.
 
@@ -475,14 +503,18 @@ class YWatchdog(YFunction):
 
         On failure, throws an exception or returns YWatchdog.TRIGGERDURATION_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YWatchdog.TRIGGERDURATION_INVALID
-        return self._triggerDuration
+        res = self._triggerDuration
+        return res
 
     def set_triggerDuration(self, newval):
         """
         Changes the duration of resets caused by the watchdog, in milliseconds.
+        Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
 
         @param newval : an integer corresponding to the duration of resets caused by the watchdog, in milliseconds
 
@@ -514,7 +546,12 @@ class YWatchdog(YFunction):
         found is returned. The search is performed first by hardware name,
         then by logical name.
 
-        @param func : a string that uniquely characterizes the watchdog
+        If a call to this object's is_online() method returns FALSE although
+        you are certain that the matching device is plugged, make sure that you did
+        call registerHub() at application initialization time.
+
+        @param func : a string that uniquely characterizes the watchdog, for instance
+                WDOGDC01.watchdog1.
 
         @return a YWatchdog object allowing you to drive the watchdog.
         """
@@ -525,9 +562,41 @@ class YWatchdog(YFunction):
             YFunction._AddToCache("Watchdog", func, obj)
         return obj
 
+    def toggle(self):
+        """
+        Switch the relay to the opposite state.
+
+        @return YAPI.SUCCESS if the call succeeds.
+
+        On failure, throws an exception or returns a negative error code.
+        """
+        # sta
+        # fw
+        # mo
+        if self._firm == 0:
+            mo = self.get_module()
+            fw = mo.get_firmwareRelease()
+            if fw == YModule.FIRMWARERELEASE_INVALID:
+                return YWatchdog.STATE_INVALID
+            self._firm = YAPI._atoi(fw)
+        if self._firm < 34921:
+            sta = self.get_state()
+            if sta == YWatchdog.STATE_INVALID:
+                return YWatchdog.STATE_INVALID
+            if sta == YWatchdog.STATE_B:
+                self.set_state(YWatchdog.STATE_A)
+            else:
+                self.set_state(YWatchdog.STATE_B)
+            return YAPI.SUCCESS
+        else:
+            return self._setAttr("state","X")
+
     def nextWatchdog(self):
         """
         Continues the enumeration of watchdog started using yFirstWatchdog().
+        Caution: You can't make any assumption about the returned watchdog order.
+        If you want to find a specific a watchdog, use Watchdog.findWatchdog()
+        and a hardwareID or a logical name.
 
         @return a pointer to a YWatchdog object, corresponding to
                 a watchdog currently online, or a None pointer
@@ -542,7 +611,7 @@ class YWatchdog(YFunction):
 
 #--- (end of YWatchdog implementation)
 
-#--- (Watchdog functions)
+#--- (YWatchdog functions)
 
     @staticmethod
     def FirstWatchdog():
@@ -576,4 +645,4 @@ class YWatchdog(YFunction):
 
         return YWatchdog.FindWatchdog(serialRef.value + "." + funcIdRef.value)
 
-#--- (end of Watchdog functions)
+#--- (end of YWatchdog functions)
